﻿Option Strict On
Imports Microsoft.Kinect

Class MainWindow

    Dim Kinect As KinectSensor

    Dim myScreenImageRect As Int32Rect
    Dim myDepthPixelData As Short()
    Dim myColorPixelData As Byte()

    Private myDepth As Double

    Property Human_Image1_Bitmap As WriteableBitmap
    Property Human_Image2_Bitmap As WriteableBitmap
    Property Room_Bitmap As WriteableBitmap


    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        myDepth = 2400
        init_kinect()
        AddHandler CompositionTarget.Rendering, Sub(renderSender As Object, renderArgs As EventArgs)
                                                    Using colorFrame As ColorImageFrame = Kinect.ColorStream.OpenNextFrame(100)
                                                        '100ミリ秒
                                                        Using depthFrame As DepthImageFrame = Kinect.DepthStream.OpenNextFrame(100)
                                                            RenderScreen(colorFrame, depthFrame)
                                                        End Using
                                                    End Using

                                                End Sub
        DataContext = Me
    End Sub

    Private Sub init_kinect()
        Try
            If KinectSensor.KinectSensors.Count = 0 Then
                MessageBox.Show("Kinectが接続されていません。")
                Exit Sub
            Else
                Kinect = KinectSensor.KinectSensors(0)
            End If

            Kinect.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30)
            Kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
            Kinect.SkeletonStream.Enable()

            Dim depthStream As DepthImageStream = Kinect.DepthStream
            myDepthPixelData = New Short(Kinect.DepthStream.FramePixelDataLength - 1) {}
            myColorPixelData = New Byte(Kinect.ColorStream.FramePixelDataLength - 1) {}


            Room_Bitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)

            Human_Image1_Bitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)
            Human_Image2_Bitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)

            myScreenImageRect = New Int32Rect(0, 0, CInt(Math.Ceiling(Human_Image2_Bitmap.Width)), CInt(Math.Ceiling(Human_Image2_Bitmap.Height)))
            Kinect.Start()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try
    End Sub

    Private Sub RenderScreen(colorFrame As ColorImageFrame, depthFrame As DepthImageFrame)
        If Kinect Is Nothing OrElse depthFrame Is Nothing OrElse colorFrame Is Nothing Then
            Return
        End If

        Dim depth As Integer = 0

        Dim depthPixelIndex As Integer

        Dim playerIndex As Integer

        Dim colorPixelIndex As Integer

        Dim colorPoint As ColorImagePoint

        Dim colorStride As Integer = colorFrame.BytesPerPixel * colorFrame.Width '2560
        Dim screenImageStride As Integer = colorFrame.BytesPerPixel * depthFrame.Width '2560

        Dim ImageIndex As Integer = 0
        Dim bytePlayer2 As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}
        Dim bytePlayer1 As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}
        Dim byteRoom As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}

        depthFrame.CopyPixelDataTo(myDepthPixelData)
        colorFrame.CopyPixelDataTo(myColorPixelData)

        For depthY As Integer = 0 To depthFrame.Height - 1
            Dim depthX As Integer = 0
            While depthX < depthFrame.Width
                depthPixelIndex = depthX + (depthY * depthFrame.Width)
                playerIndex = myDepthPixelData(depthPixelIndex) And DepthImageFrame.PlayerIndexBitmask

                colorPoint = Kinect.MapDepthToColorImagePoint(depthFrame.Format, depthX, depthY, myDepthPixelData(depthPixelIndex), colorFrame.Format)
                colorPixelIndex = (colorPoint.X * colorFrame.BytesPerPixel) + (colorPoint.Y * colorStride)

                If playerIndex <> 0 Then

                    depth = myDepthPixelData(depthPixelIndex) >> DepthImageFrame.PlayerIndexBitmaskWidth

                    If depth > myDepth Then
                        bytePlayer1(ImageIndex) = myColorPixelData(colorPixelIndex)
                        bytePlayer1(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                        bytePlayer1(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                        bytePlayer1(ImageIndex + 3) = &HFF
                    Else
                        bytePlayer2(ImageIndex) = myColorPixelData(colorPixelIndex)
                        bytePlayer2(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                        bytePlayer2(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                        bytePlayer2(ImageIndex + 3) = &HFF
                    End If
                Else
                    byteRoom(ImageIndex) = myColorPixelData(colorPixelIndex)
                    byteRoom(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                    byteRoom(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                    byteRoom(ImageIndex + 3) = &HFF
                End If
                depthX = depthX + 1
                ImageIndex = ImageIndex + colorFrame.BytesPerPixel
            End While
        Next

        'byteからビットマップへ書出し
        Human_Image1_Bitmap.WritePixels(myScreenImageRect, bytePlayer1, screenImageStride, 0)
        Human_Image2_Bitmap.WritePixels(myScreenImageRect, bytePlayer2, screenImageStride, 0)
        Room_Bitmap.WritePixels(myScreenImageRect, byteRoom, screenImageStride, 0)

        depthTextBlock.Text = String.Format("深度={0}", depth)
    End Sub

    Private Sub MainWindow_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        If Kinect Is Nothing = False Then
            If Kinect.IsRunning = True Then
                Kinect.Stop()
                Kinect.Dispose()
            End If
        End If
    End Sub

End Class


